/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 * 
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 * 
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.common.attributes;

import java.io.Serializable;

/**
 * Class AnyObjectAttributeValue represents unknown attribute values.
 * <p/>
 * This class should be used by pdp-proxy implementations to react
 * on attribute values, which they do not directly support.
 * It is recommend to create a concrete attribute value for such a 'new' type.
 *
 * @author Marko Reiprecht
 *         Created on 26.03.2007
 * @version $Revision: 2281 $
 */
public class AnyObjectAttributeValue extends AttributeValueAdapter implements Serializable {

    // ------------------------------ FIELDS ------------------------------

    /**
     * Field serialVersionUID.
     */
    private static final long serialVersionUID = -6142093848318605939L;
    /**
     * Holds a custom type name if given in the constructor, otherwise the
     * classname of the wrapped object.
     */
    protected String m_typeName;

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * Creates a new AnyObjectAttributeValue.
     *
     * @param wrappedObject the wrapped object, must be not null.
     * @throws NullPointerException if wrappedObject is null.
     * @see AttributeValueAdapter#AttributeValueAdapter(Object)
     */
    public AnyObjectAttributeValue(final Object wrappedObject) throws NullPointerException {
        this(wrappedObject, null);
    }

    /**
     * Creates a new AnyObjectAttributeValue.
     *
     * @param wrappedObject the wrapped object, must be not null.
     * @param typeName      a custom typename.
     *                      If not given or empty the class name of wrappedObject is used.
     * @throws NullPointerException if wrappedObject is null.
     * @see AttributeValueAdapter#AttributeValueAdapter(Object)
     */
    public AnyObjectAttributeValue(final Object wrappedObject, final String typeName)
            throws NullPointerException {
        super(wrappedObject);
        m_typeName = (typeName == null || typeName.length() == 0) ? super.getTypeName() : typeName;
    }

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * gets the type name of the concrete value.
     * this can be the classname of the wrapped object or
     * a custom value. It is dependend of the way it is constructed.
     *
     * @return a type name.
     * @see AttributeValue#getTypeName()
     * @see AttributeValueAdapter#getTypeName()
     */
    public String getTypeName() {
        return m_typeName;
    }
}
