/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 * 
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 * 
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.common.crypto;

import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.regex.Pattern;

/**
 * Class DigestUtil contains some convenient utility methods related to {@link java.security.MessageDigest} tasks.
 * <p/>
 * 
 * @author Marko Reiprecht
 *         Created on 19.09.2007
 * @version : $
 */
public final class DigestUtil {

    // ------------------------------ FIELDS ------------------------------

    /**
     * Field BYTE_HEX_PATTERN is the regex for bytes as two hex digits.
     */
    private static final Pattern BYTE_HEX_PATTERN = Pattern.compile(
            "^(?:(?:\\d|[abcdefABCDEF]){2})+$"
            );

    // --------------------------- CONSTRUCTORS ---------------------------

    /**
     * Constructor DigestUtil is private to ensure nobody creates instances
     * of this utility class.
     */
    private DigestUtil() {
    }

    // -------------------------- OTHER METHODS --------------------------
    /**
     * convinient mehtod, wrapping the {@link NoSuchAlgorithmException} in an {@link IllegalArgumentException} if
     * algorithm not exists.
     */
    public static MessageDigest getMessageDigest(final String algorithm) {
        try {
            return MessageDigest.getInstance(algorithm);
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalArgumentException("algorithm '" + algorithm + "' not found: " + e, e);
        }
    }

    /**
     * Method tests the both char[] on equality, ignoring the case.
     * 
     * @param a
     *            the first to check.
     * @param b
     *            the second to check.
     * @return only true if both char[] equal(ignoring the case).
     */
    public static boolean areEqualDigests(final char[] a, final char[] b) {
        if (a == b) {
            return true;
        }
        if (a == null || b == null) {
            return false;
        }
        if (a.length != b.length) {
            return false;
        }
        for (int i = 0, j = a.length; i < j; ++i) {
            if (Character.toUpperCase(a[i]) != Character.toUpperCase(b[i])) {
                return false;
            }
        }
        return true;
    }

    /**
     * Method digestToString converts a byte[] message digest to its two digit hexdecimal
     * representation as string.
     * 
     * @param digest
     *            the digest !=null.
     * @return String.
     */
    public static String digestToString(final byte[] digest) {
        return new String(digestToCharArray(digest));
    }

    /**
     * Method digestToCharArray converts a byte[] message digest to its two digit hexdecimal
     * representation as chars.
     * 
     * @param digest
     *            the digest !=null.
     * @return char[].
     */
    public static char[] digestToCharArray(final byte[] digest) {
        char[] digestchars = new char[digest.length << 1];
        for (int i = 0, j = 0, l = digest.length; i < l; ++i, ++j) {
            // convert one byte to 1 or 2 digit hex number
            String bytechars = Integer.toHexString(digest[i] & 0x00FF);
            switch (bytechars.length()) {
                case 1:
                    digestchars[j] = '0';
                    digestchars[++j] = bytechars.charAt(0);
                    break;
                default:
                    digestchars[j] = bytechars.charAt(0);
                    digestchars[++j] = bytechars.charAt(1);
                    break;
            }
        }
        return digestchars;
    }

    /**
     * Method isDigestOf test if the given byte[] array is a digest.
     * This simple tests for the correct length and is only supplied for convenience.
     * 
     * @param digest
     *            the digest !=null
     * @param expectedSize
     *            the expectedSize.
     * @return boolean.
     */
    public static boolean isDigestOf(final byte[] digest, final int expectedSize) {
        return digest.length == expectedSize;
    }

    /**
     * Method isDigestOf test if the given char[] array is a digest.
     * This tests the correct length and if all chars are in a hexadecimal range.
     * <p/>
     * The expectedSize must be the length of the digest in bytes, not in chars!
     * 
     * @param digest
     *            the digest !=null.
     * @param expectedSize
     *            the expectedSize of the digest in bytes as given by the algoritm
     *            {@link java.security.MessageDigest#getDigestLength()} Therewith this is the size in bytes, not the
     *            size of the char array,
     *            e.g. MD5 has a size of 16, and the char[] is tested for a length of 32.
     * @return boolean.
     */
    public static boolean isDigestOf(final char[] digest, final int expectedSize) {
        return digest.length == (expectedSize << 1)
                && BYTE_HEX_PATTERN.matcher(new String(digest)).matches();
    }

    /**
     * Method isDigestOf test if the given String is a digest.
     * This tests the correct length and if all chars are in a hexadecimal range.
     * <p/>
     * The expectedSize must be the length of the digest in bytes, not in chars!
     * 
     * @param digest
     *            the digest !=null.
     * @param expectedSize
     *            the expectedSize of the digest in bytes as given by the algoritm
     *            {@link java.security.MessageDigest#getDigestLength()} Therewith this is the size in bytes, not the
     *            size of the String,
     *            e.g. MD5 has a size of 16, and the String is tested for a length of 32.
     * @return boolean.
     */
    public static boolean isDigestOf(final String digest, final int expectedSize) {
        return digest.length() == (expectedSize << 1)
                && BYTE_HEX_PATTERN.matcher(digest).matches();
    }

    /**
     * Method isDigestOf test if the given byte[] array is a digest.
     * This simple tests for the correct length and is only supplied for convenience.
     * 
     * @param digest
     *            the digest !=null.
     * @param algorithm
     *            the algorithm !=null.
     * @return boolean.
     */
    public static boolean isDigestOf(final byte[] digest, final MessageDigest algorithm) {
        return isDigestOf(digest, algorithm.getDigestLength());
    }

    /**
     * Method isDigestOf test if the given char[] array is a digest.
     * This tests the correct length and if all chars are in a hexadecimal range.
     * 
     * @param digest
     *            the digest !=null.
     * @param algorithm
     *            the algorithm !=null.
     * @return boolean.
     */
    public static boolean isDigestOf(final char[] digest, final MessageDigest algorithm) {
        return isDigestOf(digest, algorithm.getDigestLength());
    }

    /**
     * Method isDigestOf test if the given char[] array is a digest.
     * This tests the correct length and if all chars are in a hexadecimal range.
     * 
     * @param digest
     *            the digest !=null.
     * @param algorithm
     *            the algorithm !=null.
     * @return boolean.
     */
    public static boolean isDigestOf(final String digest, final MessageDigest algorithm) {
        return isDigestOf(digest, algorithm.getDigestLength());
    }

    /**
     * Method isDigestOf test if the given byte[] array is a digest.
     * This simple tests for the correct length and is only supplied for convenience.
     * 
     * @param digest
     *            the digest !=null.
     * @param algorithm
     *            the algorithm !=null.
     * @return boolean.
     * @throws IllegalArgumentException
     *             if algorithm not supported.
     */
    public static boolean isDigestOf(final byte[] digest, final String algorithm) {
        return isDigestOf(digest, getMessageDigest(algorithm).getDigestLength());
    }

    /**
     * Method isDigestOf test if the given char[] array is a digest.
     * This tests the correct length and if all chars are in a hexadecimal range.
     * 
     * @param digest
     *            the digest !=null.
     * @param algorithm
     *            the method !=null.
     * @return boolean.
     * @throws java.security.IllegalArgumentException
     *             if algorithm not supported.
     */
    public static boolean isDigestOf(final char[] digest, final String algorithm) {
        return isDigestOf(digest, getMessageDigest(algorithm).getDigestLength());
    }

    /**
     * Method isDigestOf test if the given char[] array is a digest.
     * This tests the correct length and if all chars are in a hexadecimal range.
     * 
     * @param digest
     *            the digest !=null.
     * @param algorithm
     *            the method !=null.
     * @return boolean.
     * @throws IllegalArgumentException
     *             if algorithm not supported.
     */
    public static boolean isDigestOf(final String digest, final String algorithm) {
        return isDigestOf(digest, getMessageDigest(algorithm).getDigestLength());
    }

    /**
     * calcualates a digest for the given value with the given algorithm.
     * 
     * @return the byte digest.
     */
    public static final byte[] calculateDigestOf(final byte[] value, final String algorithm) {
        return getMessageDigest(algorithm).digest(value);
    }

    /**
     * calcualates a digest for the given value with the given algorithm.
     * 
     * @return the byte digest.
     */
    public static final byte[] calculateDigestOf(final char[] value, final String algorithm) {
        return calculateDigestOf(CharBuffer.wrap(value), algorithm);
    }

    /**
     * calcualates a digest for the given value with the given algorithm.
     * 
     * @return the byte digest.
     */
    public static final byte[] calculateDigestOf(final String value, final String algorithm) {
        return calculateDigestOf(CharBuffer.wrap(value), algorithm);
    }

    /**
     * calcualates a digest for the given value with the given algorithm.
     * 
     * @return the byte digest.
     */
    public static final byte[] calculateDigestOf(final CharBuffer value, final String algorithm) {
        return calculateDigestOf(Charset.forName("UTF-8").encode(value).array(), algorithm);
    }

    /**
     * Responsible for .
     * 
     * @param passwordEncryptionAlg
     * @return
     */
    public static boolean isDigestAlgorithm(final String algorithm) {
        if (algorithm == null) {
            return false;
        }
        try {
            return getMessageDigest(algorithm) != null;
        } catch (IllegalArgumentException e) {
            return false;
        }
    }

    /**
     * This is the prevered method to simple calculate a digest e.g. of password and compare it with an already known
     * password digest.
     * 
     * @param expected
     *            the expected digest as char array.
     * @param inputToCalculate
     *            the orginal password chars, from with first the digest is calculated.
     * @param algorithm
     *            the algorithm used to calculate the digest from the input.
     * @return true if equal.
     */
    public static boolean calculateAndTestForEquality(final char[] expected,
                                                      final char[] inputToCalculate,
                                                      final String algorithm) {
        return areEqualDigests(expected,
                digestToCharArray(calculateDigestOf(inputToCalculate, algorithm)));
    }

    /**
     * This is the prevered method to simple calculate a digest e.g. of password and compare it with an already known
     * password digest.
     * 
     * @param expected
     *            the expected digest as char array.
     * @param inputToCalculate
     *            the orginal password chars, from with first the digest is calculated.
     * @param algorithm
     *            the algorithm used to calculate the digest from the input.
     * @return true if equal.
     */
    public static boolean calculateAndTestForEquality(final String expected,
                                                      final String inputToCalculate,
                                                      final String algorithm) {
        return areEqualDigests(expected.toCharArray(),
                digestToCharArray(calculateDigestOf(inputToCalculate, algorithm)));
    }
}
