/*******************************************************************************
 * Copyright (C) 2009
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 * 
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 * 
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 ******************************************************************************/

package org.n52.security.common.util;

import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @see http://de.wikipedia.org/wiki/ISO_8601 for possible iso patterns.
 * @author Marko Reiprecht
 */
public final class DateUtil {

    /**
     * one hour in milliseconds.
     */
    private static final int HOUR_IN_MILLI = 1000 * 60 * 60;
    /**
     * one minute in milliseconds.
     */
    private static final int MINUTE_IN_MILLI = 1000 * 60;
    /**
     * This pattern parses the iso date time format. As example it accepts following formats:
     * <ul>
     * <li>2009-11-17T13:28:17.235+01:00 ( full date time with time zone)</li>
     * <li>2009-11-17T13:28:17.235+0100 ( full date time without timezone separator)</li>
     * <li>2009-11-17T13:28:17.235Z ( full date time in utc)</li>
     * <li>2009-11-17T13:28:17.235Z ( full date time in utc)</li>
     * <li>2009-11-17T13:28:17.235 ( full date time in utc (missing Z))</li>
     * <li>2009-11-17T13:28:17 ( date time in utc without milliseconds)</li>
     * <li>2009-11-17T13:28 ( date time in utc without milliseconds and seconds)</li>
     * <li>2009-11-17 ( date )</li>
     * <li>20091117 ( compact date)</li>
     * <li>20091117T132817.235+01:00 ( compact date time)</li>
     * <li>20091117T132817 ( compact date time in utc)</li>
     * <li>20091117132817 ( compact date time in utc)</li>
     * </ul>
     * Groups:
     * <ul>
     * <li>1 year</li>
     * <li>2 month</li>
     * <li>3 day of month</li>
     * <li>4 hour of day</li>
     * <li>5 minute of hour</li>
     * <li>6 second of minute</li>
     * <li>7 millisecond of second</li>
     * <li>8 timezone sign (+|-)</li>
     * <li>9 timezone hour</li>
     * <li>10 timezone minute of hour</li>
     * </ul>
     */
    private static final Pattern ISO_PARSE_PATTERN =
        Pattern.compile("^(\\d{4})-?(\\d{2})-?(\\d{2})(?:T?(\\d{2}):?(\\d{2}):?(\\d{2})?(?:\\.(\\d+))?(?:(?:(\\+|-)(\\d{2}):?(\\d{2}))|Z?))?$");

    /**
     * Creates a new CalendarInstance in the UTC time zone for the given time settings.
     * 
     * @param year
     *            the year
     * @param month
     *            the month (starting with index 1 (this is not like the
     *            {@link Calendar#set(int, int, int, int, int, int)} method).
     * @param dayofMonth
     *            the day of month.
     * @param hour
     *            the hour.
     * @param min
     *            the min.
     * @param sec
     *            the sec.
     * @param milli
     *            the milli.
     * @return a new calendar.
     */
    public static Calendar createUTCCalendar(final int year,
                                             final int month,
                                             final int dayofMonth,
                                             final int hour,
                                             final int min,
                                             final int sec,
                                             final int milli) {
        return createCalendar(TimeZone.getTimeZone("UTC"), year, month, dayofMonth, hour, min, sec, milli);
    }

    /**
     * Creates a new CalendarInstance in the given time zone for the given time settings.
     * 
     * @param zone
     *            time zone.
     * @param year
     *            the year
     * @param month
     *            the month (starting with index 1 (this is not like the
     *            {@link Calendar#set(int, int, int, int, int, int)} method).
     * @param dayofMonth
     *            the day of month.
     * @param hour
     *            the hour.
     * @param min
     *            the min.
     * @param sec
     *            the sec.
     * @param milli
     *            the milli.
     * @return a new calendar.
     */
    public static Calendar createCalendar(final TimeZone zone,
                                          final int year,
                                          final int month,
                                          final int dayofMonth,
                                          final int hour,
                                          final int min,
                                          final int sec,
                                          final int milli) {
        Calendar cal = Calendar.getInstance(zone);
        cal.set(year, month - 1, dayofMonth, hour, min, sec);
        cal.set(Calendar.MILLISECOND, milli);
        return cal;
    }

    /**
     * Creates UTC Timezone with given offset in millisec.
     */
    public static TimeZone createUTCTimeZone(int offset) {
        TimeZone zone = TimeZone.getTimeZone("UTC");
        if (offset != 0) {
            zone.setRawOffset(offset);
        }
        return zone;
    }

    /**
     * Creates UTC Timezone with given hour , minute offset.
     * the minutes are always added to the absolute hour value,
     * but the sign of the hour value is used to calculate sign
     * of the offset.
     * 
     * @param hour
     *            the hour offset.
     * @param minutes
     *            the minutes in the hour.
     * @return a UTC Timezone.
     */
    public static TimeZone createUTCTimeZone(int hour, int minutes) {
        int offset = (hour < 0 ? -1 : 1) * (Math.abs(hour) * HOUR_IN_MILLI + minutes * MINUTE_IN_MILLI);
        return createUTCTimeZone(offset);
    }

    /**
     * Method parses a iso formated date.
     * e.g: 2009-11-17T13:28:17+0100 (with timezone offset)
     * or 2009-11-17T13:28:17Z (as UTC time)
     * 
     * @param isoStr
     *            the iso format string.
     * @return Date the parsed date.
     * @throws IllegalArgumentException
     *             if isoStr can't be parsed.
     */
    public static Calendar fromISOString(final String isoStr) throws IllegalArgumentException {
        if (isoStr == null) {
            throw new IllegalArgumentException("<isoStr> must not null!");
        }
        Matcher m = ISO_PARSE_PATTERN.matcher(isoStr);
        if (!m.matches()) {
            throw new IllegalArgumentException(
                    "<isoStr> does not match the ISO 8601 datetime format (e.g. yyyy-MM-dd'T'HH:mm:ss.SSS'Z')!");
        }
        // creates timezone from timezone offset (e.g. +01:00)
        TimeZone zone =
            createUTCTimeZone(
                (chk(m.group(8), "+").equals("+") ? 1 : -1) *
                toInt(chk(m.group(9), "0")),
                toInt(chk(m.group(10), "0")));

        return createCalendar(zone,
                toInt(chk(m.group(1), "0")),
                toInt(chk(m.group(2), "0")),
                toInt(chk(m.group(3), "0")),
                toInt(chk(m.group(4), "0")),
                toInt(chk(m.group(5), "0")),
                toInt(chk(m.group(6), "0")),
                toInt(chk(m.group(7), "0")));
    }

    /**
     * Method formats a date to an iso formated string.
     * It will always produce the utc iso format, e.g.: 2009-11-17T13:28:17.235Z.
     * 
     * @param date
     * @return
     */
    public static String toUTCISOString(final Calendar date) {
        return toUTCISOString(date.getTime());
    }

    /**
     * Method formats a date to an iso formated string.
     * It will always produce the utc iso format, e.g.: 2009-11-17T13:28:17.235Z.
     */
    public static String toUTCISOString(final Date date) {
        TimeZone zone = TimeZone.getTimeZone("UTC");
        Calendar cal = Calendar.getInstance(zone);
        cal.setTime(date);
        return tofullISOString(cal);
    }

    /**
     * Method formats a date to an iso formated string.
     * It will always produce the full iso format, e.g.: 2009-11-17T13:28:17.235+0100.
     * 
     * @param date
     *            the date.
     * @return String.
     */
    public static String tofullISOString(final Date date) {
        TimeZone zone = TimeZone.getTimeZone("UTC");
        zone.setRawOffset(-date.getTimezoneOffset() * 60 * 1000);
        Calendar cal = Calendar.getInstance(zone);
        cal.setTime(date);
        return tofullISOString(cal);
    }

    /**
     * Method formats a date to an iso formated string.
     * It will always produce the full iso format, e.g.: 2009-11-17T13:28:17.235+0100.
     * 
     * @param date
     *            the date.
     * @return String.
     */
    public static String tofullISOString(final Calendar date) {
        StringBuffer timeStr = new StringBuffer(29);
        appendTime(timeStr, date.get(Calendar.YEAR), 4);
        timeStr.append("-");
        appendTime(timeStr, date.get(Calendar.MONTH) + 1, 2);
        timeStr.append("-");
        appendTime(timeStr, date.get(Calendar.DAY_OF_MONTH), 2);
        timeStr.append("T");
        appendTime(timeStr, date.get(Calendar.HOUR_OF_DAY), 2);
        timeStr.append(":");
        appendTime(timeStr, date.get(Calendar.MINUTE), 2);
        timeStr.append(":");
        appendTime(timeStr, date.get(Calendar.SECOND), 2);
        int milli = date.get(Calendar.MILLISECOND);
        if (milli != 0) {
            timeStr.append(".");
            appendTime(timeStr, milli, 0);
        }
        int timezoneOffset = date.getTimeZone().getOffset(date.getTimeInMillis());
        if (timezoneOffset == 0) {
            timeStr.append("Z");
        } else {
            timeStr.append(timezoneOffset < 0 ? "-" : "+");
            timezoneOffset = Math.abs(timezoneOffset);
            appendTime(timeStr, timezoneOffset / HOUR_IN_MILLI, 2);
            appendTime(timeStr, (timezoneOffset % HOUR_IN_MILLI) / MINUTE_IN_MILLI, 2);
        }
        return timeStr.toString();
    }

    /**
     * Responsible for appending a time value.
     * 
     * @param timeStr
     * @param i
     * @param j
     */
    private static void appendTime(final StringBuffer timeStr, int time, int numbercount) {
        String val = String.valueOf(time);
        if (numbercount > 0) {
            for (int i = val.length(); i < numbercount; ++i) {
                timeStr.append("0");
            }
        }
        timeStr.append(val);
    }

    /**
     * Responsible for parsing the given String to an int.
     * 
     * @param val
     *            the value.
     * @return the parsed int.
     */
    private static int toInt(String val) {
        return Integer.parseInt(val);
    }

    /**
     * checks test for null or empty.
     * if true def is returned, otherwise the test value.
     * 
     * @param test
     *            the test.
     * @param def
     *            the default.
     * @return test | def.
     */
    private static String chk(final String test, final String def) {
        if (test == null || test.length() == 0) {
            return def;
        }
        return test;
    }
}
