package org.n52.security.common.util;

import java.net.MalformedURLException;
import java.net.URL;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Compares URI's that may contain the wild card character '*' or '**' characters. Thus
 * it works like the PathWildcardMatcher but has some special behaviors for URLs.
 * If the pattern or incomming values can't be parsed to URLs it will fallback to the normal PathWildcardMatchFunction
 * behavior.
 * <ul>
 * <li>http://host/enforcement/myEP/{asterisk}/layers/Countries ==
 * http://host/enforcement/myEP/httpauth/layers/Countries</li>
 * </ul>
 * 
 * @author Marko Reiprecht
 */
public class URLWildcardMatcher {
    private static final Log LOG = LogFactory.getLog(URLWildcardMatcher.class);

    private PathWildcardMatcher m_protocolMatcher;
    private URL m_wildCardURI;
    private PathWildcardMatcher m_hostMatcher;
    private PathWildcardMatcher m_queryMatcher;
    private PathWildcardMatcher m_pathMatcher;
    private PathWildcardMatcher m_fullURLMatcher;

    /**
     * Creates new instance.
     */
    public URLWildcardMatcher(final String wildCardUrlString, final boolean caseInsensitive) {
        try {
            m_wildCardURI = new URL(wildCardUrlString);
            m_protocolMatcher = new PathWildcardMatcher(m_wildCardURI.getProtocol(), caseInsensitive);
            m_hostMatcher = new PathWildcardMatcher(m_wildCardURI.getHost(), caseInsensitive);
            m_pathMatcher = new PathWildcardMatcher(m_wildCardURI.getPath(), caseInsensitive);
            if (m_wildCardURI.getQuery() != null) {
                m_queryMatcher = new PathWildcardMatcher(m_wildCardURI.getQuery(), caseInsensitive);
            }
        } catch (MalformedURLException e) {
            LOG.warn(
                    "'" + wildCardUrlString +
                    "' is not a valid URL, so only check for normal regexp match.");
        }
        m_fullURLMatcher = new PathWildcardMatcher(wildCardUrlString, caseInsensitive);
    }

    public boolean matches(final String testString) {
        if (m_wildCardURI != null) {
            try {
                // be strict if uris
                URL test = new URL(testString);
                // Compare base (scheme, host, port) before comparing path
                if (!m_protocolMatcher.matches(test.getProtocol())) {
                    return false;
                }
                if (!m_hostMatcher.matches(test.getHost())) {
                    return false;
                }
                if (!equalsPorts(m_wildCardURI, test)) {
                    return false;
                }
                // now we must carefull because of the ? char between path and query
                if (m_wildCardURI.getQuery() != null) {
                    return m_pathMatcher.matches(test.getPath()) &&
                            m_queryMatcher.matches(test.getQuery());
                } else if (test.getQuery() == null) {
                    // wild card did not have a query but maybe a wild card on the end of the name!
                    return m_pathMatcher.matches(test.getPath());
                } else {
                    return m_pathMatcher.matches(test.getPath() + '?' + test.getQuery());
                }
            } catch (MalformedURLException e) {
                LOG.warn("'" + testString + "' is not a valid URL, so only check for normal regexp match.");
            }
        }
        // match full url string
        return m_fullURLMatcher.matches(testString);
    }

    private boolean equalsPorts(URL uri1, URL uri2) {
        int port1 = uri1.getPort() != -1 ? uri1.getPort() : uri1.getDefaultPort();
        int port2 = uri2.getPort() != -1 ? uri2.getPort() : uri2.getDefaultPort();
        return port1 == port2;
    }
}
