/*******************************************************************************
 * Copyright (C) 2008
 * by 52 North Initiative for Geospatial Open Source Software GmbH
 *
 * Contact: Andreas Wytzisk
 * 52 North Initiative for Geospatial Open Source Software GmbH
 * Martin-Luther-King-Weg 24
 * 48155 Muenster, Germany
 * info@52north.org
 *
 * This program is free software; you can redistribute and/or modify it under
 * the terms of the GNU General Public License version 2 as published by the
 * Free Software Foundation.
 *
 * This program is distributed WITHOUT ANY WARRANTY; even without the implied
 * WARRANTY OF MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program (see gnu-gpl v2.txt). If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA or
 * visit the Free Software Foundation web page, http://www.fsf.org.
 *
 * Author: Marko Reiprecht
 * Created: 24.03.2008
 ******************************************************************************/
/* $Id: $ */
package org.n52.security.common.xml;

/**
 * Class DOMParserOptions are options describing some common options for DOMParsers.
 *
 * @author Marko Reiprecht
 *         Created on 24.03.2008
 * @version $Revision: $
 */
public class DOMParserOptions {

    // ------------------------------ FIELDS ------------------------------

    private boolean m_namespaceaware;
    private boolean m_validating;
    private boolean m_ignoreComments;
    private boolean m_ignoringElementContentWhitespace;

    // --------------------- GETTER / SETTER METHODS ---------------------

    /**
     * Method gets the default options.
     * The default is a namespaceaware, non validating parser, which ignores comments.
     *
     * @return DOMParserOptions.
     */
    public static DOMParserOptions getDefault() {
        return new DOMParserOptions()
                .namespaceaware()
                .notValidating()
                .notIgnoreComments()
                .notIgnoreElementContentWhitespace();
    }

    /**
     * Method sets the ignoreComments to false.
     *
     * @return DOMParserOptions.
     */
    public DOMParserOptions notIgnoreComments() {
        m_ignoreComments = false;
        return this;
    }

    /**
     * Method gets it comments are ignored.
     *
     * @return boolean.
     */
    public boolean isIgnoreComments() {
        return m_ignoreComments;
    }

    /**
     * Method sets the ignore comments flag.
     *
     * @param ignorecomments the ignoreComments.
     */
    public void setIgnoreComments(final boolean ignorecomments) {
        m_ignoreComments = ignorecomments;
    }

    /**
     * Method gets the parser is namespaceaware.
     *
     * @return boolean.
     */
    public boolean isNamespaceaware() {
        return m_namespaceaware;
    }

    /**
     * Method sets the namespaceaware flag.
     *
     * @param namespaceaware the namespaceaware.
     */
    public void setNamespaceaware(final boolean namespaceaware) {
        m_namespaceaware = namespaceaware;
    }

    /**
     * Method gets if the parser is in validating mode.
     * Consider that validating works only in xml files based on dtds.
     * To validate xml files based on xml schemata use the schema class.
     *
     * @return boolean.
     */
    public boolean isValidating() {
        return m_validating;
    }

    /**
     * Method sets the validating flag.
     *
     * @param validating the validating.
     */
    public void setValidating(final boolean validating) {
        m_validating = validating;
    }

    // ------------------------ CANONICAL METHODS ------------------------

    /**
     * Method equals.
     *
     * @param o the o.
     * @return boolean.
     */
    public boolean equals(final Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof DOMParserOptions)) {
            return false;
        }

        final DOMParserOptions that = (DOMParserOptions) o;

        if (m_ignoreComments != that.m_ignoreComments) {
            return false;
        }
        if (m_namespaceaware != that.m_namespaceaware) {
            return false;
        }
        return m_validating == that.m_validating;
    }

    /**
     * Method hashCode.
     *
     * @return int.
     */
    public int hashCode() {
        int result;
        result = (m_namespaceaware ? 1 : 0);
        result = 31 * result + (m_validating ? 1 : 0);
        result = 31 * result + (m_ignoreComments ? 1 : 0);
        result = 31 * result + (m_ignoringElementContentWhitespace ? 1 : 0);
        return result;
    }

    /**
     * Method toString.
     *
     * @return String.
     */
    public String toString() {
        final StringBuffer sb = new StringBuffer();
        sb.append("org.n52.security.common.xml.DOMParserOptions");
        sb.append("{isNamespaceaware=").append(isNamespaceaware());
        sb.append(", isValidating=").append(isValidating());
        sb.append(", isIgnoreComments=").append(isIgnoreComments());
        sb.append(", IgnoringElementContentWhitespace=").append(isIgnoringElementContentWhitespace());
        sb.append('}');
        return sb.toString();
    }

    // -------------------------- OTHER METHODS --------------------------

    /**
     * Method sets the ignoreComments flag.
     *
     * @return DOMParserOptions.
     */
    public DOMParserOptions ignoreComments() {
        m_ignoreComments = true;
        return this;
    }

    /**
     * Method sets the namespaceaware flag.
     *
     * @return DOMParserOptions.
     */
    public DOMParserOptions namespaceaware() {
        m_namespaceaware = true;
        return this;
    }

    /**
     * Method sets the namespaceaware flag to false.
     *
     * @return DOMParserOptions.
     */
    public DOMParserOptions notNamespaceaware() {
        m_namespaceaware = false;
        return this;
    }

    /**
     * Method sets the validating to false.
     *
     * @return DOMParserOptions.
     */
    public DOMParserOptions notValidating() {
        m_validating = false;
        return this;
    }

    /**
     * Method sets the validating flag.
     *
     * @return DOMParserOptions.
     */
    public DOMParserOptions validating() {
        m_validating = true;
        return this;
    }

    /**
     * @return
     */
    public boolean isIgnoringElementContentWhitespace() {
        return m_ignoringElementContentWhitespace;
    }

    /**
     * @return
     */
    public DOMParserOptions ignoreElementContentWhitespace() {
        m_ignoringElementContentWhitespace = true;
        return this;
    }


    /**
     * @return
     */
    public DOMParserOptions notIgnoreElementContentWhitespace() {
        m_ignoringElementContentWhitespace = false;
        return this;
    }

    public void setIgnoreElementContentWhitespace(boolean ignoreElementWhitespace) {
        m_ignoringElementContentWhitespace = ignoreElementWhitespace;
    }
}
